@@ -416,12 +416,6 @@ private:
 		return s1.st_dev == s2.st_dev && s1.st_ino == s2.st_ino && s1.st_rdev == s2.st_rdev;
 	}
 
-	int reportBusyException(request_rec *r) {
-		ap_custom_response(r, HTTP_SERVICE_UNAVAILABLE,
-			"This website is too busy right now.  Please try again later.");
-		return HTTP_SERVICE_UNAVAILABLE;
-	}
-
 	/**
 	 * Gather some information about the request and do some preparations.
 	 *
@@ -1382,10 +1376,6 @@ public:
 		config["turbocaching"] = serverConfig.turbocaching;
 		config["old_routing"] = serverConfig.oldRouting;
 		config["prestart_urls"] = strsetToJson(serverConfig.prestartURLs);
-		config["admin_panel_url"] = nonEmptyString(serverConfig.adminPanelUrl);
-		config["admin_panel_auth_type"] = nonEmptyString(serverConfig.adminPanelAuthType);
-		config["admin_panel_username"] = nonEmptyString(serverConfig.adminPanelUsername);
-		config["admin_panel_password"] = nonEmptyString(serverConfig.adminPanelPassword);
 		config["disable_log_prefix"] = serverConfig.disableLogPrefix;
 
 		if (!serverConfig.logFile.empty()) {
