@@ -35,7 +35,6 @@
 #include <netdb.h>
 #include <string>
 #include <utility>
-#include <vector>
 #include <oxt/macros.hpp>
 #include <StaticString.h>
 #include <FileDescriptor.h>
@@ -44,16 +43,15 @@ namespace Passenger {
 
 using namespace std;
 
+
+/****** Server address types support ******/
+
 enum ServerAddressType {
 	SAT_UNIX,
 	SAT_TCP,
 	SAT_UNKNOWN
 };
 
-typedef ssize_t (*WritevFunction)(int fildes, const struct iovec *iov, int iovcnt);
-
-bool purgeStdio(FILE *f);
-
 /**
  * Accepts a server address in one of the following formats, and returns which one it is:
  *
@@ -96,47 +94,8 @@ void parseTcpSocketAddress(const StaticString & restrict_ref address,
  */
 bool isLocalSocketAddress(const StaticString &address);
 
-/**
- * Sets a socket in blocking mode.
- *
- * @throws SystemException Something went wrong.
- * @ingroup Support
- */
-void setBlocking(int fd);
 
-/**
- * Sets a socket in non-blocking mode.
- *
- * @throws SystemException Something went wrong.
- * @ingroup Support
- */
-void setNonBlocking(int fd);
-
-/**
- * Try to call the Linux accept4() system call. If the system call is
- * not available, then -1 is returned and errno is set to ENOSYS.
- */
-int callAccept4(int sock,
-	struct sockaddr * restrict addr,
-	socklen_t * restrict addr_len,
-	int options);
-
-/**
- * Resolves the given host name and returns a list of IP addresses.
- * `hostname` may also be an IP address, in which case it is
- * returned. You may explicitly specify a `port` as a hint to
- * the DNS resolver; set to 0 if you don't care or can't provide a
- * port number.
- *
- * If `shuffle` is set, and the host name resolves to multiple
- * IP addresses, then these addresses will be shuffled before they are
- * returned in order to improve load balancing.
- *
- * @throws IOException DNS resolution failure.
- */
-vector<string> resolveHostname(const string &hostname,
-	unsigned int port = 0,
-	bool shuffle = true);
+/****** Server socket creation ******/
 
 /**
  * Create a new Unix or TCP server socket, depending on the address type.
@@ -157,7 +116,7 @@ vector<string> resolveHostname(const string &hostname,
  * @throws boost::thread_interrupted A system call has been interrupted.
  * @ingroup Support
  */
-int createServer(const StaticString &address,
+ int createServer(const StaticString &address,
 	unsigned int backlogSize = 0,
 	bool autoDelete = true,
 	const char *file = __FILE__,
@@ -209,6 +168,9 @@ int createTcpServer(const char *address = "0.0.0.0",
 	const char *file = __FILE__,
 	unsigned int line = __LINE__);
 
+
+/****** Socket connection establishment (blocking) ******/
+
 /**
  * Connect to a server at the given address in a blocking manner.
  *
@@ -224,7 +186,7 @@ int createTcpServer(const char *address = "0.0.0.0",
  * @throws boost::thread_interrupted A system call has been interrupted.
  * @ingroup Support
  */
-int connectToServer(const StaticString &address, const char *file,
+ int connectToServer(const StaticString &address, const char *file,
 	unsigned int line);
 
 /**
@@ -260,130 +222,120 @@ int connectToUnixServer(const StaticString &filename, const char *file,
 int connectToTcpServer(const StaticString &hostname, unsigned int port,
 	const char *file, unsigned int line);
 
-/** State structure for non-blocking connectToUnixServer(). */
-struct NUnix_State {
-	FileDescriptor fd;
-	string filename;
-};
+
+/****** Socket connection establishment (non-blocking) ******/
 
 /**
- * Setup a Unix domain socket for non-blocking connecting. When done,
- * the file descriptor can be accessed through <tt>state.fd</tt>.
+ * Establish a socket connection in non-blocking mode.
+ *
+ * @param address An address as accepted by getSocketAddressType().
+ * @param file The name of the source file that called this function,
+ *             for file descriptor logging purposes.
+ * @param line The line in the source file that called this function.
+ * @return The file descriptor of the connected client socket, plus whether
+ *   it was immediately connected (true) or pending (false). If pending,
+ *   the caller should poll() on the socket until it's writable.
+ * @throws ArgumentException Something went wrong.
+ * @throws IOException Something went wrong.
+ * @throws SystemException Something went wrong.
+ * @throws boost::thread_interrupted A system call has been interrupted.
+ */
+std::pair<int, bool> createNonBlockingSocketConnection(const StaticString &address, const char *file, unsigned int line);
+
+/**
+ * Establish a Unix domain socket connection in non-blocking mode.
  *
- * @param state A state structure.
  * @param filename The filename of the socket to connect to.
  * @param file The name of the source file that called this function,
  *             for file descriptor logging purposes.
  * @param line The line in the source file that called this function.
- * @throws SystemException Something went wrong.
- * @throws boost::thread_interrupted A system call has been interrupted.
- * @ingroup Support
- */
-void setupNonBlockingUnixSocket(NUnix_State & restrict_ref state,
-	const StaticString & restrict_ref filename, const char *file,
-	unsigned int line);
-
-/**
- * Connect a Unix domain socket in non-blocking mode.
- *
- * @param state A state structure.
- * @return True if the socket was successfully connected, false if the socket isn't
- *         ready yet, in which case the caller should select() on the socket until it's writable.
- * @throws RuntimeException Something went wrong.
+ * @return The file descriptor of the connected client socket, plus whether
+ *   it was immediately connected (true) or pending (false). If pending,
+ *   the caller should poll() on the socket until it's writable.
+ * @throws ArgumentException Something went wrong.
  * @throws SystemException Something went wrong while connecting to the Unix server.
  * @throws boost::thread_interrupted A system call has been interrupted.
- * @ingroup Support
  */
-bool connectToUnixServer(NUnix_State &state);
-
-/** State structure for non-blocking connectToTcpServer(). */
-struct NTCP_State {
-	FileDescriptor fd;
-	struct addrinfo hints, *res;
-	string hostname;
-	int port;
-
-	NTCP_State() {
-		memset(&hints, 0, sizeof(hints));
-		res = NULL;
-		port = 0;
-	}
-
-	~NTCP_State() {
-		if (res != NULL) {
-			freeaddrinfo(res);
-		}
-	}
-};
+std::pair<int, bool> createNonBlockingUnixSocketConnection(const StaticString &filename, const char *file, unsigned int line);
 
 /**
- * Setup a TCP socket for non-blocking connecting. When done,
- * the file descriptor can be accessed through <tt>state.fd</tt>.
+ * Establish a TCP socket connection in non-blocking mode.
  *
- * @param state A state structure.
  * @param hostname The host name of the TCP server.
  * @param port The port number of the TCP server.
  * @param file The name of the source file that called this function,
  *             for file descriptor logging purposes.
  * @param line The line in the source file that called this function.
+ * @return The file descriptor of the connected client socket, plus whether
+ *   it was immediately connected (true) or pending (false). If pending,
+ *   the caller should poll() on the socket until it's writable.
  * @throws IOException Something went wrong.
  * @throws SystemException Something went wrong.
  * @throws boost::thread_interrupted A system call has been interrupted.
- * @ingroup Support
  */
-void setupNonBlockingTcpSocket(NTCP_State & restrict_ref state,
-	const StaticString & restrict_ref hostname,
-	int port, const char *file, unsigned int line);
+std::pair<int, bool> createNonBlockingTcpSocketConnection(const StaticString &hostname, unsigned int port, const char *file, unsigned int line);
 
-/**
- * Connect a TCP socket in non-blocking mode.
- *
- * @param state A state structure.
- * @return True if the socket was successfully connected, false if the socket isn't
- *         ready yet, in which case the caller should select() on the socket until it's writable.
- * @throws SystemException Something went wrong while connecting to the server.
- * @throws boost::thread_interrupted A system call has been interrupted.
- * @ingroup Support
- */
-bool connectToTcpServer(NTCP_State &state);
 
-struct NConnect_State {
-	ServerAddressType type;
-	NUnix_State s_unix;
-	NTCP_State s_tcp;
+/****** Scope guards ******/
+
+/** RAII construct for ensuring that a file descriptor gets closed at scope exit. */
+class FdGuard {
+private:
+	int mFd = -1;
+
+public:
+	FdGuard() { }
+	FdGuard(const FdGuard &other) = delete;
+	FdGuard(FdGuard &&other);
+	FdGuard(int fd, const char *sourceFile, unsigned int sourceLine);
+	/** @throws SystemException File descriptor close error. If exception is already being thrown, then only warns. */
+	~FdGuard() noexcept(false);
+
+	FdGuard &operator=(const FdGuard &other) = delete;
+	FdGuard &operator=(FdGuard &&other);
+
+	/** Don't close file descriptor at object destruction. */
+	void clear() noexcept;
+
+	/**
+	 * Close file descriptor now. Idempotent.
+	 *
+	 * @throws SystemException
+	 */
+	void runNow() noexcept(false);
 };
 
-/**
- * Setup a socket for non-blocking connecting to the given address.
- *
- * @param A state structure.
- * @param address An address as accepted by getSocketAddressType().
- * @param file The name of the source file that called this function,
- *             for file descriptor logging purposes.
- * @param line The line in the source file that called this function.
- * @throws ArgumentException Unknown address type.
- * @throws RuntimeException Something went wrong.
- * @throws SystemException Something went wrong.
- * @throws IOException Something went wrong.
- * @throws boost::thread_interrupted A system call has been interrupted.
- * @ingroup Support
- */
-void setupNonBlockingSocket(NConnect_State & restrict_ref state,
-	const StaticString & restrict_ref address, const char *file,
-	unsigned int line);
+
+/****** Other ******/
+
+typedef ssize_t (*WritevFunction)(int fildes, const struct iovec *iov, int iovcnt);
+
+bool purgeStdio(FILE *f);
 
 /**
- * Connect a socket in non-blocking mode.
+ * Sets a socket in blocking mode.
  *
- * @param state A state structure.
- * @return True if the socket was successfully connected, false if the socket isn't
- *         ready yet, in which case the caller should select() on the socket until it's writable.
- * @throws RuntimeException Something went wrong.
  * @throws SystemException Something went wrong.
- * @throws boost::thread_interrupted A system call has been interrupted.
  * @ingroup Support
  */
-bool connectToServer(NConnect_State &state);
+void setBlocking(int fd);
+
+/**
+ * Sets a socket in non-blocking mode.
+ *
+ * @throws SystemException Something went wrong.
+ * @ingroup Support
+ */
+void setNonBlocking(int fd);
+
+/**
+ * Try to call the Linux accept4() system call. If the system call is
+ * not available, then -1 is returned and errno is set to ENOSYS.
+ */
+int callAccept4(int sock,
+	struct sockaddr * restrict addr,
+	socklen_t * restrict addr_len,
+	int options);
 
 /**
  * Checks whether the given TCP server is connectable. Because this check
@@ -392,6 +344,7 @@ bool connectToServer(NConnect_State &state);
  * deducted from the `*timeout` value. A timeout of 100000 microseconds is
  * recommended for most use cases.
  *
+ * @throws ArgumentException Something went wrong.
  * @throws IOException Something went wrong.
  * @throws SystemException Something went wrong.
  * @throws boost::thread_interrupted A system call has been interrupted.
@@ -451,9 +404,11 @@ bool waitUntilReadable(int fd, unsigned long long *timeout);
 bool waitUntilWritable(int fd, unsigned long long *timeout);
 
 /**
- * Attempts to read exactly <tt>size</tt> bytes of data from the given file
- * descriptor, and put the result in <tt>buf</tt>. On non-blocking sockets
- * this function will block by poll()ing the socket.
+ * Attempts to read exactly `size` bytes of data, putting the result in `buf`.
+ * This is in contrast to read(), which may read less than `size` bytes even
+ * when EOF is not reached.
+ *
+ * On non-blocking sockets, this function blocks by poll()ing the socket.
  *
  * @param buf The buffer to place the read data in. This buffer must be at least
  *            <tt>size</tt> bytes long.
