@@ -1,8 +1,8 @@
 #  Phusion Passenger - https://www.phusionpassenger.com/
-#  Copyright (c) 2010-2018 Phusion Holding B.V.
+#  Copyright (c) 2010-2025 Asynchronous B.V.
 #
 #  "Passenger", "Phusion Passenger" and "Union Station" are registered
-#  trademarks of Phusion Holding B.V.
+#  trademarks of Asynchronous B.V.
 #
 #  Permission is hereby granted, free of charge, to any person obtaining a copy
 #  of this software and associated documentation files (the "Software"), to deal
@@ -72,15 +72,11 @@ module PhusionPassenger
 
     def self.run_compiler(description, command, source_file, source, capture_output = false)
       if verbose?
-        message = "#{description}\n" <<
-          "Running: #{command}\n"
+        message = "#{description}\nRunning: #{command}\n"
         if source.strip.empty?
-          message << "Source file is empty."
+          message += "Source file is empty."
         else
-          message << "Source file contains:\n" <<
-            "-------------------------\n" <<
-            unindent(source) <<
-            "\n-------------------------"
+          message += "Source file contains:\n-------------------------\n#{unindent(source)}\n-------------------------"
         end
         log(message)
       end
@@ -92,10 +88,7 @@ module PhusionPassenger
           result = nil
           exec_error_reason = e.message
         end
-        log("Output:\n" <<
-          "-------------------------\n" <<
-          output.to_s <<
-          "\n-------------------------")
+        log("Output:\n-------------------------\n#{output}\n-------------------------")
       elsif verbose?
         result = system(command)
       else
@@ -481,6 +474,13 @@ module PhusionPassenger
     end
     memoize :cxx_block_support_ok?, true
 
+    def self.cxx_supports_wno_vla_cxx_extension_flag?
+      return try_compile_with_warning_flag(
+        "Checking for C++ compiler '-Wno-vla-cxx-extension' support",
+        :cxx, '', '-Wno-vla-cxx-extension')
+    end
+    memoize :cxx_supports_wno_vla_cxx_extension_flag?, true
+
     # Returns whether compiling C++ with -fvisibility=hidden might result
     # in tons of useless warnings, like this:
     # http://code.google.com/p/phusion-passenger/issues/detail?id=526
@@ -508,7 +508,7 @@ module PhusionPassenger
     def self.address_sanitizer_flags
       if cc_is_clang?
         if `#{cc} --help` =~ /-fsanitize=/
-          "-fsanitize=address -fsanitize-address-use-after-return=always -fsanitize-address-use-after-scope"
+          "-fsanitize=address -fsanitize-address-use-after-return=always"
         else
           "-faddress-sanitizer"
         end
@@ -525,25 +525,21 @@ module PhusionPassenger
       end
     end
 
-    def self.cxx_11_flag
-      # C++11 support on FreeBSD 10.0 + Clang seems to be bugged.
-      # http://llvm.org/bugs/show_bug.cgi?id=18310
-      return nil if os_name_simple == "freebsd"
-
+    def self.cxx_14_flag
       source = %{
         struct Foo {
           Foo(Foo &&f) { }
         };
       }
-      if try_compile("Checking for C++ -std=gnu++11 compiler flag", :cxx, source, '-std=gnu++11')
-        return "-std=gnu++11"
-      elsif try_compile("Checking for C++ -std=c++11 compiler flag", :cxx, source, '-std=c++11')
-        return "-std=c++11"
+      if try_compile("Checking for C++ -std=gnu++14 compiler flag", :cxx, source, '-std=gnu++14')
+        return "-std=gnu++14"
+      elsif try_compile("Checking for C++ -std=c++14 compiler flag", :cxx, source, '-std=c++14')
+        return "-std=c++14"
       else
         return nil
       end
     end
-    memoize :cxx_11_flag, true
+    memoize :cxx_14_flag, true
 
     def self.has_rt_library?
       return try_link("Checking for -lrt support",
@@ -563,12 +559,6 @@ module PhusionPassenger
     end
     memoize :has_dl_library?, true
 
-    def self.has_alloca_h?
-      return try_compile("Checking for alloca.h",
-        :c, '#include <alloca.h>')
-    end
-    memoize :has_alloca_h?, true
-
     def self.has_accept4?
       return try_compile("Checking for accept4()", :c, %Q{
         #define _GNU_SOURCE
@@ -591,7 +581,7 @@ module PhusionPassenger
         result = '-g'
       end
       if cc_supports_fno_limit_debug_info_flag?
-        result << ' -fno-limit-debug-info'
+        result += ' -fno-limit-debug-info'
       end
       result
     end
@@ -603,13 +593,13 @@ module PhusionPassenger
       # -ggdb instead.
       #
       # In any case we'll always want to use -ggdb for better GDB debugging.
-      if cc_is_gcc?
+      if cxx_is_gcc?
         result = '-ggdb'
       else
         result = '-g'
       end
       if cxx_supports_fno_limit_debug_info_flag?
-        result << ' -fno-limit-debug-info'
+        result += ' -fno-limit-debug-info'
       end
       result
     end
