@@ -1,11 +1,13 @@
 #include <TestSupport.h>
 #include <limits>
+#include <cassert>
 #include <Constants.h>
 #include <IOTools/IOUtils.h>
 #include <IOTools/BufferedIO.h>
 #include <IOTools/MessageIO.h>
 #include <Core/ApplicationPool/TestSession.h>
 #include <Core/Controller.h>
+#include <ev.h>
 
 using namespace std;
 using namespace boost;
@@ -13,37 +15,95 @@ using namespace Passenger;
 using namespace Passenger::Core;
 
 namespace tut {
+	/*
+	 * How this test suite works:
+	 *
+	 * 1. Initialization:
+	 *    Call init() early, which creates a controller and starts a background event loop.
+	 *
+	 * 2. Mocking sessions:
+	 *    Use useTestSessionObject() to mock the session object that the the controller
+	 *    receives from ApplicationPool.
+	 *
+	 * 3. Testing requests to the controller:
+	 *    1. Establish a connection with connectToServer().
+	 *    2. Use sendRequest() to send an HTTP requests.
+	 *    3. Optional: call waitUntilSessionInitiated() to wait until the controller
+	 *       has initiated a session. This adds an extra check so that if the test fails
+	 *       then you at least know whether the problem is before or after session initiation.
+	 *    4. Use readPeerRequestHeader() to read the HTTP response.
+	 *    5. Perform assertions on peerRequestHeader, which contains the HTTP response header.
+	 */
 	struct Core_ControllerTest: public TestBase {
-		class MyController: public Core::Controller {
+		/*
+		 * Works just like the normal Core::Controller, but allows the `appPool->asyncGet()`
+		 * result as well as the session socket connect timeout to be mocked by assigning the corresponding fields.
+		 * It also allows causing the session socket to delay connecting forever by setting forceSessionSocketConnectTimeout.
+		 */
+		class TestController: public Core::Controller {
 		protected:
 			virtual void asyncGetFromApplicationPool(Request *req,
 				ApplicationPool2::GetCallback callback)
+				override
 			{
-				callback(sessionToReturn, exceptionToReturn);
-				sessionToReturn.reset();
+				// If this assertion fails then it means one of:
+				// - You didn't call `mockNextSession()`, or:
+				// - The controller called `appPool->asyncGet()` too many times.
+				//   Remember that `mockNextSession()` only mocks the session object
+				//   for the specified number of calls. There's probably
+				//   something deeper wrong here, so increase log level to figure out
+				//   what's going on. One example is an EVENTUALLY timeout causing a
+				//   varying number of calls to asyncGet to happen, and you mocked too few.
+				assert(mockSession != nullptr || mockException != nullptr);
+				callback(mockSession, mockException);
+				if (--mockedSessionCount == 0) {
+					mockSession.reset();
+					mockException.reset();
+				}
+			}
+
+			virtual int getSessionSocketConnectIoWatchConditions() const override {
+				if (forceSessionSocketConnectTimeout) {
+					return EV_READ;
+				} else {
+					return Controller::getSessionSocketConnectIoWatchConditions();
+				}
+			}
+
+			virtual double getSessionSocketEffectiveConnectTimeout(Request *req) const override {
+				if (forceSessionSocketConnectTimeout) {
+					return req->connectTimeout / 1000.0;
+				} else {
+					return Controller::getSessionSocketEffectiveConnectTimeout(req);
+				}
 			}
 
 		public:
-			ApplicationPool2::AbstractSessionPtr sessionToReturn;
-			ApplicationPool2::ExceptionPtr exceptionToReturn;
+			ApplicationPool2::AbstractSessionPtr mockSession;
+			ApplicationPool2::ExceptionPtr mockException;
+			unsigned int mockedSessionCount;
+			bool forceSessionSocketConnectTimeout;
 
-			MyController(ServerKit::Context *context,
+			TestController(ServerKit::Context *context,
 				const Core::ControllerSchema &schema,
 				const Json::Value &initialConfig,
 				const Core::ControllerSingleAppModeSchema &singleAppModeSchema,
 				const Json::Value &singleAppModeConfig)
 				: Core::Controller(context, schema, initialConfig, ConfigKit::DummyTranslator(),
-					&singleAppModeSchema, &singleAppModeConfig, ConfigKit::DummyTranslator())
+				   &singleAppModeSchema, &singleAppModeConfig, ConfigKit::DummyTranslator()),
+				  mockedSessionCount(0),
+				  forceSessionSocketConnectTimeout(false)
 				{ }
 		};
 
+		// Dependencies
 		BackgroundEventLoop bg;
 		ServerKit::Schema skSchema;
 		ServerKit::Context context;
 		WrapperRegistry::Registry wrapperRegistry;
 		Core::ControllerSchema schema;
 		Core::ControllerSingleAppModeSchema singleAppModeSchema;
-		MyController *controller;
+		TestController *controller;
 		SpawningKit::Context::Schema skContextSchema;
 		SpawningKit::Context skContext;
 		SpawningKit::FactoryPtr spawningKitFactory;
@@ -51,7 +111,11 @@ namespace tut {
 		PoolPtr appPool;
 		Json::Value config, singleAppModeConfig;
 		int serverSocket;
+
+		// Mock objects
 		TestSession testSession;
+
+		// Connection and response objects
 		FileDescriptor clientConnection;
 		BufferedIO clientConnectionIO;
 		string peerRequestHeader;
@@ -106,11 +170,11 @@ namespace tut {
 			LoggingKit::setLevel(LoggingKit::CRIT);
 			clientConnection.close();
 			if (controller != NULL) {
-				bg.safe->runSync(boost::bind(&MyController::shutdown, controller, true));
-				while (getServerState() != MyController::FINISHED_SHUTDOWN) {
+				bg.safe->runSync([&] { controller->shutdown(true); });
+				while (getServerState() != TestController::FINISHED_SHUTDOWN) {
 					syscalls::usleep(10000);
 				}
-				bg.safe->runSync(boost::bind(&Core_ControllerTest::destroyController, this));
+				bg.safe->runSync([this] { delete controller; });
 			}
 			safelyClose(serverSocket);
 			unlink("tmp.server");
@@ -119,16 +183,12 @@ namespace tut {
 
 		void startLoop() {
 			if (!bg.isStarted()) {
-				bg.start();
+				bg.start("Core_ControllerTest Background Loop");
 			}
 		}
 
-		void destroyController() {
-			delete controller;
-		}
-
 		void init() {
-			controller = new MyController(&context, schema, config,
+			controller = new TestController(&context, schema, config,
 				singleAppModeSchema, singleAppModeConfig);
 			controller->resourceLocator = resourceLocator;
 			controller->wrapperRegistry = &wrapperRegistry;
@@ -158,47 +218,44 @@ namespace tut {
 			ensure_equals(getTotalBytesConsumed(), totalBytesConsumed + data.size());
 		}
 
-		void useTestSessionObject() {
-			bg.safe->runSync(boost::bind(&Core_ControllerTest::_setTestSessionObject, this));
+		/**
+		 * Ensures that the next time the controller calls `appPool->asyncGet()`, it gets `testSession`
+		 * instead of a real Session.
+		 *
+		 * Note that this only works for the next `sessions` number of invocations of `appPool->asyncGet()`.
+		 * Subsequent calls get nullptr unless you call `mockNextSession()` again before that happens.
+		 */
+		void mockNextSession(unsigned int sessions = 1) {
+			bg.safe->runSync([&] {
+				controller->mockedSessionCount = sessions;
+				controller->mockSession.reset(&testSession, false);
+			});
 		}
 
-		void _setTestSessionObject() {
-			controller->sessionToReturn.reset(&testSession, false);
-		}
-
-		MyController::State getServerState() {
+		TestController::State getServerState() {
 			Controller::State result;
-			bg.safe->runSync(boost::bind(&Core_ControllerTest::_getServerState,
-				this, &result));
+			bg.safe->runSync([&] { result = controller->serverState; });
 			return result;
 		}
 
-		void _getServerState(MyController::State *state) {
-			*state = controller->serverState;
+		unsigned int getRemainingMockedRequests() {
+			unsigned int result;
+			bg.safe->runSync([&] { result = controller->mockedSessionCount; });
+			return result;
 		}
 
 		Json::Value inspectStateAsJson() {
 			Json::Value result;
-			bg.safe->runSync(boost::bind(&Core_ControllerTest::_inspectStateAsJson,
-				this, &result));
+			bg.safe->runSync([&] { result = controller->inspectStateAsJson(); });
 			return result;
 		}
 
-		void _inspectStateAsJson(Json::Value *result) {
-			*result = controller->inspectStateAsJson();
-		}
-
 		unsigned long long getTotalBytesConsumed() {
 			unsigned long long result;
-			bg.safe->runSync(boost::bind(&Core_ControllerTest::_getTotalBytesConsumed,
-				this, &result));
+			bg.safe->runSync([&] { result = controller->totalBytesConsumed; });
 			return result;
 		}
 
-		void _getTotalBytesConsumed(unsigned long long *result) {
-			*result = controller->totalBytesConsumed;
-		}
-
 		string readPeerRequestHeader(string *peerRequestHeader = NULL) {
 			if (peerRequestHeader == NULL) {
 				peerRequestHeader = &this->peerRequestHeader;
@@ -306,7 +363,7 @@ namespace tut {
 		set_test_name("Session protocol: request URI");
 
 		init();
-		useTestSessionObject();
+		mockNextSession();
 
 		connectToServer();
 		sendRequest(
@@ -325,7 +382,7 @@ namespace tut {
 		set_test_name("HTTP protocol: request URI");
 
 		init();
-		useTestSessionObject();
+		mockNextSession();
 		testSession.setProtocol("http_session");
 
 		connectToServer();
@@ -350,7 +407,7 @@ namespace tut {
 			" and forwards the raw data");
 
 		init();
-		useTestSessionObject();
+		mockNextSession();
 
 		connectToServer();
 		sendRequest(
@@ -380,7 +437,7 @@ namespace tut {
 			" and forwards the chunked data");
 
 		init();
-		useTestSessionObject();
+		mockNextSession();
 
 		connectToServer();
 		sendRequest(
@@ -415,7 +472,7 @@ namespace tut {
 			" and forwards the raw data");
 
 		init();
-		useTestSessionObject();
+		mockNextSession();
 
 		connectToServer();
 		sendRequest(
@@ -457,7 +514,7 @@ namespace tut {
 			" and forwards the raw data");
 
 		init();
-		useTestSessionObject();
+		mockNextSession();
 
 		connectToServer();
 		sendRequest(
@@ -483,7 +540,7 @@ namespace tut {
 			" it forwards the chunked data");
 
 		init();
-		useTestSessionObject();
+		mockNextSession();
 
 		connectToServer();
 		sendRequest(
@@ -517,7 +574,7 @@ namespace tut {
 			" it forwards the raw data");
 
 		init();
-		useTestSessionObject();
+		mockNextSession();
 
 		connectToServer();
 		sendRequest(
@@ -557,7 +614,7 @@ namespace tut {
 		set_test_name("Fixed response body");
 
 		init();
-		useTestSessionObject();
+		mockNextSession();
 
 		connectToServer();
 		sendRequest(
@@ -584,7 +641,7 @@ namespace tut {
 		set_test_name("Response body until EOF");
 
 		init();
-		useTestSessionObject();
+		mockNextSession();
 
 		connectToServer();
 		sendRequest(
@@ -610,7 +667,7 @@ namespace tut {
 		set_test_name("Chunked response body");
 
 		init();
-		useTestSessionObject();
+		mockNextSession();
 
 		connectToServer();
 		sendRequest(
@@ -642,7 +699,7 @@ namespace tut {
 		set_test_name("Upgraded response body");
 
 		init();
-		useTestSessionObject();
+		mockNextSession();
 
 		connectToServer();
 		sendRequest(
@@ -672,7 +729,7 @@ namespace tut {
 		set_test_name("Perform keep-alive on application responses that allow it");
 
 		init();
-		useTestSessionObject();
+		mockNextSession();
 
 		connectToServer();
 		sendRequest(
@@ -698,7 +755,7 @@ namespace tut {
 		set_test_name("Don't perform keep-alive on application responses that don't allow it");
 
 		init();
-		useTestSessionObject();
+		mockNextSession();
 
 		connectToServer();
 		sendRequest(
@@ -725,7 +782,7 @@ namespace tut {
 		set_test_name("Don't perform keep-alive if an error occurred");
 
 		init();
-		useTestSessionObject();
+		mockNextSession();
 
 		connectToServer();
 		sendRequest(
@@ -758,7 +815,7 @@ namespace tut {
 			" application connection");
 
 		init();
-		useTestSessionObject();
+		mockNextSession();
 
 		connectToServer();
 		sendRequest(
@@ -788,7 +845,7 @@ namespace tut {
 			" application connection");
 
 		init();
-		useTestSessionObject();
+		mockNextSession();
 
 		connectToServer();
 		sendRequest(
@@ -819,7 +876,7 @@ namespace tut {
 			" application connection");
 
 		init();
-		useTestSessionObject();
+		mockNextSession();
 
 		connectToServer();
 		sendRequest(
@@ -850,7 +907,7 @@ namespace tut {
 			" application connection");
 
 		init();
-		useTestSessionObject();
+		mockNextSession();
 
 		connectToServer();
 		sendRequest(
@@ -882,7 +939,7 @@ namespace tut {
 			" early read error and does not keep-alive the application connection");
 
 		init();
-		useTestSessionObject();
+		mockNextSession();
 		testSession.setProtocol("http_session");
 
 		connectToServer();
@@ -912,7 +969,7 @@ namespace tut {
 			" early read error and does not keep-alive the application connection");
 
 		init();
-		useTestSessionObject();
+		mockNextSession();
 		testSession.setProtocol("http_session");
 
 		connectToServer();
@@ -945,7 +1002,7 @@ namespace tut {
 			" and does not keep-alive the application connection");
 
 		init();
-		useTestSessionObject();
+		mockNextSession();
 		testSession.setProtocol("http_session");
 
 		connectToServer();
@@ -979,7 +1036,7 @@ namespace tut {
 			" application connection");
 
 		init();
-		useTestSessionObject();
+		mockNextSession();
 		testSession.setProtocol("http_session");
 
 		connectToServer();
@@ -1013,7 +1070,7 @@ namespace tut {
 			" we still try to send a 502 (which shouldn't log warn)");
 
 		init();
-		useTestSessionObject();
+		mockNextSession();
 
 		connectToServer();
 		sendRequest(
@@ -1040,7 +1097,7 @@ namespace tut {
 			" we still try to send a 502 (which shouldn't log warn)");
 
 		init();
-		useTestSessionObject();
+		mockNextSession();
 		testSession.setProtocol("http_session");
 
 		connectToServer();
@@ -1062,12 +1119,12 @@ namespace tut {
 	}
 
 	TEST_METHOD(55) {
-		set_test_name("Session protocol: if application decides not to "
-			" finish the response (close), and the client is still there "
+		set_test_name("Session protocol: if application decides not to"
+			" finish the response (close), and the client is still there"
 			" we should send a 502 (which should log warn)");
 
 		init();
-		useTestSessionObject();
+		mockNextSession();
 
 		connectToServer();
 		sendRequest(
@@ -1091,12 +1148,12 @@ namespace tut {
 	}
 
 	TEST_METHOD(56) {
-		set_test_name("HTTP protocol: if application decides not to "
-			" finish the response (close), and the client is still there "
+		set_test_name("HTTP protocol: if application decides not to"
+			" finish the response (close), and the client is still there"
 			" we should send a 502 (which should log warn)");
 
 		init();
-		useTestSessionObject();
+		mockNextSession();
 		testSession.setProtocol("http_session");
 
 		connectToServer();
@@ -1119,4 +1176,62 @@ namespace tut {
 		string header = readResponseHeader();
 		ensure(containsSubstring(header, "HTTP/1.1 502"));
 	}
+
+	/***** Application connection non-instant connect *****/
+
+	TEST_METHOD(60) {
+		set_test_name("If app connection is not instantly established,"
+			" then it waits until establishment finishes");
+
+		init();
+		mockNextSession();
+		testSession.forceNonInstantConnect();
+
+		connectToServer();
+		sendRequest(
+			"GET /hello HTTP/1.1\r\n"
+			"Host: localhost\r\n"
+			"Connection: close\r\n"
+			"\r\n");
+		waitUntilSessionInitiated();
+
+		readPeerRequestHeader();
+		sendPeerResponse(
+			"HTTP/1.1 200 OK\r\n"
+			"Content-Type: text/plain\r\n"
+			"Content-Length: 2\r\n\r\n"
+			"ok");
+
+		waitUntilSessionClosed();
+		ensure("(1)", testSession.isSuccessful());
+	}
+
+	TEST_METHOD(61) {
+		set_test_name("If app connection establishment keeps timing out,"
+			" then it returns a 504 Gateway Timeout error");
+
+		init();
+		mockNextSession(Controller::MAX_SESSION_CHECKOUT_TRY);
+		testSession.forceNonInstantConnect();
+		controller->forceSessionSocketConnectTimeout = true;
+
+		connectToServer();
+		sendRequest(
+			"GET /hello HTTP/1.1\r\n"
+			"!~: \r\n"
+			"!~PASSENGER_APP_CONNECT_TIMEOUT: 1\r\n" // fast for EVENTUALLY timeout below
+			"!~: \r\n"
+			"Host: localhost\r\n"
+			"Connection: close\r\n"
+			"\r\n");
+
+		EVENTUALLY(5,
+			result = getRemainingMockedRequests() == 0;
+		);
+		// We'll want to assert that the Core Controller responds with a timeout error.
+		waitUntilSessionClosed();
+		ensure_not("(1)", testSession.isSuccessful());
+		string header = readResponseHeader();
+		ensure(containsSubstring(header, "HTTP/1.1 504 Gateway Timeout"));
+	}
 }
